@@ -8,11 +8,8 @@
 #include <linux/if_vlan.h>
 #include <linux/if_ether.h>
 
-struct wireguard_device;
-struct endpoint;
-
-int socket_init(struct wireguard_device *wg);
-void socket_uninit(struct wireguard_device *wg);
+int socket_init(struct wireguard_device *wg, u16 port);
+void socket_reinit(struct wireguard_device *wg, struct sock *new4, struct sock *new6);
 int socket_send_buffer_to_peer(struct wireguard_peer *peer, void *data, size_t len, u8 ds);
 int socket_send_skb_to_peer(struct wireguard_peer *peer, struct sk_buff *skb, u8 ds);
 int socket_send_buffer_as_reply_to_skb(struct wireguard_device *wg, struct sk_buff *in_skb, void *out_buffer, size_t len);
@@ -27,7 +24,7 @@ void socket_clear_peer_endpoint_src(struct wireguard_peer *peer);
 	struct endpoint __endpoint; \
 	socket_endpoint_from_skb(&__endpoint, skb); \
 	net_dbg_ratelimited(fmt, dev, &__endpoint.addr, ##__VA_ARGS__); \
-} while(0)
+} while (0)
 #else
 #define net_dbg_skb_ratelimited(fmt, skb, ...)
 #endif
